@@ -1,6 +1,6 @@
 /****************************************************************************
 
- $Id: kstartupinfo.cpp,v 1.59.2.1 2004/10/12 14:58:19 lunakl Exp $
+ $Id: kstartupinfo.cpp 700792 2007-08-16 12:57:23Z mueller $
 
  Copyright (C) 2001-2003 Lubos Lunak        <l.lunak@kde.org>
 
@@ -33,7 +33,7 @@ DEALINGS IN THE SOFTWARE.
 #include <qwidget.h>
 
 #include "config.h"
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
 //#ifdef Q_WS_X11 // FIXME(E): Re-implement in a less X11 specific way
 #include <qglobal.h>
 #ifdef HAVE_CONFIG_H
@@ -51,21 +51,22 @@ DEALINGS IN THE SOFTWARE.
 #include <sys/time.h>
 #include <stdlib.h>
 #include <qtimer.h>
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
-#include <netwm.h> 
+#ifdef Q_WS_X11
+#include <netwm.h>
 #endif
 #include <kdebug.h>
 #include <kapplication.h>
 #include <signal.h>
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
-#include <kwinmodule.h> 
-#include <kxmessages.h> 
+#ifdef Q_WS_X11
+#include <kwinmodule.h>
+#include <kxmessages.h>
 #include <kwin.h>
 #endif
 
 static const char* const NET_STARTUP_MSG = "_NET_STARTUP_INFO";
 static const char* const NET_STARTUP_WINDOW = "_NET_STARTUP_ID";
-// DESKTOP_STARTUP_ID is used also in kinit/wrapper.c
+// DESKTOP_STARTUP_ID is used also in kinit/wrapper.c ,
+// kdesu in both kdelibs and kdebase and who knows where else
 static const char* const NET_STARTUP_ENV = "DESKTOP_STARTUP_ID";
 
 static bool auto_app_started_sending = true;
@@ -83,9 +84,9 @@ class KStartupInfo::Data
     : public KStartupInfoData
     {
     public:
-        Data() {}; // just because it's in a QMap
+        Data() : KStartupInfoData(), age(0) {} // just because it's in a QMap
         Data( const QString& txt_P )
-            : KStartupInfoData( txt_P ), age( 0 ) {};
+            : KStartupInfoData( txt_P ), age( 0 ) {}
         unsigned int age;
     };
 
@@ -97,7 +98,7 @@ struct KStartupInfoPrivate
         QMap< KStartupInfoId, KStartupInfo::Data > silent_startups;
         // contains ASN's that had change: but no new: yet
         QMap< KStartupInfoId, KStartupInfo::Data > uninited_startups;
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
         KWinModule* wm_module;
         KXMessages msgs;
 #endif
@@ -105,7 +106,7 @@ struct KStartupInfoPrivate
 	int flags;
 	KStartupInfoPrivate( int flags_P )
     	    :
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
 	    msgs( NET_STARTUP_MSG, NULL, false ),
 #endif
 	      flags( flags_P ) {}
@@ -134,7 +135,7 @@ void KStartupInfo::init( int flags_P )
         return;
 
     d = new KStartupInfoPrivate( flags_P );
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
     if( !( d->flags & DisableKWinModule ))
         {
         d->wm_module = new KWinModule( this );
@@ -145,7 +146,7 @@ void KStartupInfo::init( int flags_P )
         d->wm_module = NULL;
     connect( &d->msgs, SIGNAL( gotMessage( const QString& )), SLOT( got_message( const QString& )));
 #endif
-    d->cleanup = new QTimer( this );
+    d->cleanup = new QTimer( this, "cleanup" );
     connect( d->cleanup, SIGNAL( timeout()), SLOT( startups_cleanup()));
     }
 
@@ -261,7 +262,7 @@ void KStartupInfo::new_startup_info_internal( const KStartupInfoId& id_P,
 	    emit gotNewStartup( id_P, d->startups[ id_P ] );
 	    return;
 	    }
-        emit gotStartupChange( id_P, d->startups[ id_P ] );
+        emit gotStartupChange( id_P, d->silent_startups[ id_P ] );
         return;
         }
     if( d->uninited_startups.contains( id_P ))
@@ -509,7 +510,7 @@ void KStartupInfo::appStarted( const QCString& startup_id )
         KStartupInfo::sendFinish( id );
     else if( getenv( "DISPLAY" ) != NULL ) // don't rely on qt_xdisplay()
         {
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
         Display* disp = XOpenDisplay( NULL );
         if( disp != NULL )
             {
@@ -558,11 +559,14 @@ void KStartupInfo::setNewStartupId( QWidget* window, const QCString& startup_id
                 }
             }
         if( activate )
+            {
+            KWin::setOnDesktop( window->winId(), KWin::currentDesktop());
         // This is not very nice, but there's no way how to get any
         // usable timestamp without ASN, so force activating the window.
         // And even with ASN, it's not possible to get the timestamp here,
         // so if the WM doesn't have support for ASN, it can't be used either.
             KWin::forceActiveWindow( window->winId());
+            }
         }
     KStartupInfo::handleAutoAppStartedSending();
     }
@@ -613,7 +617,7 @@ KStartupInfo::startup_t KStartupInfo::check_startup_internal( WId w_P, KStartupI
             }
         return find_id( id, id_O, data_O ) ? Match : NoMatch;
         }
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
     NETWinInfo info( qt_xdisplay(),  w_P, qt_xrootwin(),
         NET::WMWindowType | NET::WMPid | NET::WMState );
     pid_t pid = info.pid();
@@ -730,7 +734,7 @@ bool KStartupInfo::find_wclass( QCString res_name, QCString res_class,
     return false;
     }
 
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
 static Atom net_startup_atom = None;
 
 static QCString read_startup_id_property( WId w_P )
@@ -756,7 +760,7 @@ static QCString read_startup_id_property( WId w_P )
 
 QCString KStartupInfo::windowStartupId( WId w_P )
     {
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
     if( net_startup_atom == None )
         net_startup_atom = XInternAtom( qt_xdisplay(), NET_STARTUP_WINDOW, False );
     if( utf8_string_atom == None )
@@ -778,7 +782,7 @@ QCString KStartupInfo::windowStartupId( WId w_P )
 
 void KStartupInfo::setWindowStartupId( WId w_P, const QCString& id_P )
     {
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
     if( id_P.isNull())
         return;
     if( net_startup_atom == None )
@@ -792,7 +796,7 @@ void KStartupInfo::setWindowStartupId( WId w_P, const QCString& id_P )
 
 QCString KStartupInfo::get_window_hostname( WId w_P )
     {
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
     XTextProperty tp;
     char** hh;
     int cnt;
@@ -854,7 +858,7 @@ void KStartupInfo::startups_cleanup_internal( bool age_P )
             {
             const KStartupInfoId& key = it.key();
             ++it;
-            kdDebug( 172 ) << "entry timeout:" << key.id() << endl;
+            kdDebug( 172 ) << "startups entry timeout:" << key.id() << endl;
             remove_startup_info_internal( key );
             }
         else
@@ -873,7 +877,7 @@ void KStartupInfo::startups_cleanup_internal( bool age_P )
             {
             const KStartupInfoId& key = it.key();
             ++it;
-            kdDebug( 172 ) << "entry timeout:" << key.id() << endl;
+            kdDebug( 172 ) << "silent entry timeout:" << key.id() << endl;
             remove_startup_info_internal( key );
             }
         else
@@ -892,7 +896,7 @@ void KStartupInfo::startups_cleanup_internal( bool age_P )
             {
             const KStartupInfoId& key = it.key();
             ++it;
-            kdDebug( 172 ) << "entry timeout:" << key.id() << endl;
+            kdDebug( 172 ) << "uninited entry timeout:" << key.id() << endl;
             remove_startup_info_internal( key );
             }
         else
@@ -930,7 +934,7 @@ QCString KStartupInfo::createNewStartupId()
     hostname[ 0 ] = '\0';
     if (!gethostname( hostname, 255 ))
 	hostname[sizeof(hostname)-1] = '\0';
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
     extern Time qt_x_user_time;
 #else
     long qt_x_user_time = 0;
@@ -944,7 +948,7 @@ QCString KStartupInfo::createNewStartupId()
 
 struct KStartupInfoIdPrivate
     {
-    KStartupInfoIdPrivate() : id( "" ) {};
+    KStartupInfoIdPrivate() : id( "" ) {}
     QCString id; // id
     };
 
@@ -994,7 +998,7 @@ void KStartupInfoId::initId( const QCString& id_P )
         }
     d->id = KStartupInfo::createNewStartupId();
     }
-    
+
 bool KStartupInfoId::setupStartupEnv() const
     {
     if( id().isEmpty())
@@ -1101,7 +1105,7 @@ unsigned long KStartupInfoId::timestamp() const
 struct KStartupInfoDataPrivate
     {
     KStartupInfoDataPrivate() : desktop( 0 ), wmclass( "" ), hostname( "" ),
-	silent( KStartupInfoData::Unknown ), timestamp( -1U ), screen( -1 ) {};
+	silent( KStartupInfoData::Unknown ), timestamp( -1U ), screen( -1 ) {}
     QString bin;
     QString name;
     QString description;
@@ -1130,7 +1134,7 @@ QString KStartupInfoData::to_text() const
         ret += QString::fromLatin1( " DESKTOP=%1" )
             .arg( d->desktop == NET::OnAllDesktops ? NET::OnAllDesktops : d->desktop - 1 ); // spec counts from 0
     if( !d->wmclass.isEmpty())
-        ret += QString::fromLatin1( " WMCLASS=%1" ).arg( d->wmclass );
+        ret += QString::fromLatin1( " WMCLASS=\"%1\"" ).arg( d->wmclass );
     if( !d->hostname.isEmpty())
         ret += QString::fromLatin1( " HOSTNAME=%1" ).arg( d->hostname );
     for( QValueList< pid_t >::ConstIterator it = d->pids.begin();
