@@ -5,6 +5,7 @@
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Peter Kelly (pmk@post.com)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
+ *           (C) 2006 Allan Sandfeld Jensen (kde@carewolf.com)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,20 +19,23 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 //#define EVENT_DEBUG
 #include "dom/dom_exception.h"
 #include "dom/dom_node.h"
+#include "dom/html_image.h"
 #include "xml/dom_textimpl.h"
 #include "xml/dom_docimpl.h"
 #include "xml/dom2_eventsimpl.h"
 #include "xml/dom_elementimpl.h"
+#include "xml/dom_restyler.h"
 
 #include "html/dtd.h"
 #include "html/htmlparser.h"
+#include "html/html_imageimpl.h"
 
 #include "rendering/render_canvas.h"
 #include "misc/htmlhashes.h"
@@ -53,7 +57,7 @@
 using namespace DOM;
 using namespace khtml;
 
-AttrImpl::AttrImpl(ElementImpl* element, DocumentPtr* docPtr, NodeImpl::Id attrId,
+AttrImpl::AttrImpl(ElementImpl* element, DocumentImpl* docPtr, NodeImpl::Id attrId,
 		   DOMStringImpl *value, DOMStringImpl *prefix)
     : NodeBaseImpl(docPtr),
       m_element(element),
@@ -164,12 +168,17 @@ void AttrImpl::setValue( const DOMString &v, int &exceptioncode )
     if (m_value == v.implementation())
 	return;
 
+    if (m_element && m_attrId == ATTR_ID)
+        m_element->updateId(m_value, v.implementation());
+
     m_value->deref();
     m_value = v.implementation();
     m_value->ref();
 
-    if (m_element)
+    if (m_element) {
         m_element->parseAttribute(m_attrId,m_value);
+        m_element->attributeChanged(m_attrId);
+    }
 }
 
 void AttrImpl::setNodeValue( const DOMString &v, int &exceptioncode )
@@ -240,6 +249,21 @@ void AttrImpl::setElement(ElementImpl *element)
     m_element = element;
 }
 
+// Strictly speaking, these two methods should not be needed, but
+// we can't fully deal with the mess that are DOM attributes right..
+DOMStringImpl* AttrImpl::textContent() const
+{
+    if (m_value)
+        return new DOMStringImpl(m_value->s, m_value->l);
+    else
+        return 0;
+}
+
+void AttrImpl::setTextContent( const DOMString &text, int& exceptioncode )
+{
+    setValue(text, exceptioncode);
+}
+
 // -------------------------------------------------------------------------
 
 void AttributeImpl::setValue(DOMStringImpl *value, ElementImpl *element)
@@ -248,11 +272,18 @@ void AttributeImpl::setValue(DOMStringImpl *value, ElementImpl *element)
     if (m_attrId) {
 	if (m_data.value == value)
 	    return;
+
+	if (element && m_attrId == ATTR_ID)
+	   element->updateId(m_data.value, value);
+
 	m_data.value->deref();
 	m_data.value = value;
 	m_data.value->ref();
-	if (element)
+
+	if (element) {
 	    element->parseAttribute(this);
+	    element->attributeChanged(m_attrId);
+        }
     }
     else {
 	int exceptioncode = 0;
@@ -261,12 +292,12 @@ void AttributeImpl::setValue(DOMStringImpl *value, ElementImpl *element)
     }
 }
 
-AttrImpl *AttributeImpl::createAttr(ElementImpl *element, DocumentPtr *docPtr)
+AttrImpl *AttributeImpl::createAttr(ElementImpl *element, DocumentImpl *docPtr)
 {
     if (m_attrId) {
 	AttrImpl *attr = new AttrImpl(element,docPtr,m_attrId,m_data.value);
         if (!attr) return 0;
-        attr->setHTMLCompat( docPtr->document()->htmlMode() != DocumentImpl::XHtml );
+        attr->setHTMLCompat( docPtr->htmlMode() != DocumentImpl::XHtml );
 	m_data.value->deref();
 	m_data.attr = attr;
 	m_data.attr->ref();
@@ -289,7 +320,7 @@ void AttributeImpl::free()
 
 // -------------------------------------------------------------------------
 
-ElementImpl::ElementImpl(DocumentPtr *doc)
+ElementImpl::ElementImpl(DocumentImpl *doc)
     : NodeBaseImpl(doc)
 {
     namedAttrMap = 0;
@@ -319,22 +350,27 @@ unsigned short ElementImpl::nodeType() const
     return Node::ELEMENT_NODE;
 }
 
-DOMString ElementImpl::getAttribute( NodeImpl::Id id, bool nsAware, const DOMString& qName) const
+DOMStringImpl* ElementImpl::getAttributeImpl( NodeImpl::Id id, bool nsAware, DOMStringImpl* qName) const
 {
     if (!namedAttrMap)
-	return DOMString();
+	return 0;
 
-    DOMStringImpl *value = namedAttrMap->getValue(id, nsAware, (qName.isEmpty() ? 0: qName.implementation()));
+    DOMStringImpl *value = namedAttrMap->getValue(id, nsAware, qName);
     if (value)
 	return value;
 
     // then search in default attr in case it is not yet set
     NamedAttrMapImpl* dm = defaultMap();
-    value = dm ? dm->getValue(id, nsAware, (qName.isEmpty() ? 0: qName.implementation())) : 0;
+    value = dm ? dm->getValue(id, nsAware, qName) : 0;
     if (value)
 	return value;
 
-    return DOMString();
+    return 0;
+}
+
+DOMString ElementImpl::getAttribute( NodeImpl::Id id, bool nsAware, const DOMString& qName) const
+{
+    return DOMString(getAttributeImpl(id, nsAware, qName.implementation()));
 }
 
 void ElementImpl::setAttribute(NodeImpl::Id id, const DOMString &value, const DOMString& qName, int &exceptioncode)
@@ -376,6 +412,15 @@ void ElementImpl::setAttribute(NodeImpl::Id id, const DOMString &value)
 
 void ElementImpl::setAttributeMap( NamedAttrMapImpl* list )
 {
+    // If setting the whole map changes the id attribute, we need to
+    // call updateId.
+    DOMStringImpl *oldId = namedAttrMap ? namedAttrMap->getValue(ATTR_ID) : 0;
+    DOMStringImpl *newId = list ? list->getValue(ATTR_ID) : 0;
+
+    if (oldId || newId) {
+       updateId(oldId, newId);
+    }
+
     if (namedAttrMap) {
 	namedAttrMap->detachFromElement();
         namedAttrMap->deref();
@@ -388,8 +433,10 @@ void ElementImpl::setAttributeMap( NamedAttrMapImpl* list )
         assert(namedAttrMap->m_element == 0);
         namedAttrMap->setElement(this);
         unsigned long len = namedAttrMap->length();
-        for (unsigned long i = 0; i < len; i++)
+        for (unsigned long i = 0; i < len; i++) {
             parseAttribute(&namedAttrMap->m_attrs[i]);
+            attributeChanged(namedAttrMap->m_attrs[i].id());
+        }
     }
 }
 
@@ -463,6 +510,22 @@ NamedAttrMapImpl* ElementImpl::defaultMap() const
     return 0;
 }
 
+RenderStyle *ElementImpl::styleForRenderer(RenderObject * /*parentRenderer*/)
+{
+    return getDocument()->styleSelector()->styleForElement(this);
+}
+
+RenderObject *ElementImpl::createRenderer(RenderArena *arena, RenderStyle *style)
+{
+    if (getDocument()->documentElement() == this && style->display() == NONE) {
+        // Ignore display: none on root elements.  Force a display of block in that case.
+        RenderBlock* result = new (arena) RenderBlock(this);
+        if (result) result->setStyle(style);
+        return result;
+    }
+    return RenderObject::createObject(this, style);
+}
+
 void ElementImpl::attach()
 {
     assert(!attached());
@@ -470,21 +533,69 @@ void ElementImpl::attach()
     assert(parentNode());
 
 #if SPEED_DEBUG < 1
-    if (parentNode()->renderer() && parentNode()->renderer()->childAllowed()) {
-        RenderStyle* _style = getDocument()->styleSelector()->styleForElement(this);
-        _style->ref();
-        m_render = RenderObject::createObject(this, _style);
-        if(m_render) {
-            m_render->setStyle(_style);
-            parentNode()->renderer()->addChild(m_render, nextRenderer());
-        }
-        _style->deref();
-    }
+    createRendererIfNeeded();
 #endif
 
     NodeBaseImpl::attach();
 }
 
+void ElementImpl::close()
+{
+    NodeImpl::close();
+
+    // Trigger all the addChild changes as one large dynamic appendChildren change
+    if (attached())
+        backwardsStructureChanged();
+}
+
+void ElementImpl::detach()
+{
+    getDocument()->dynamicDomRestyler().resetDependencies(this);
+
+    NodeBaseImpl::detach();
+}
+
+void ElementImpl::structureChanged()
+{
+    NodeBaseImpl::structureChanged();
+
+    if (!getDocument()->renderer())
+        return; // the document is about to be destroyed
+
+    getDocument()->dynamicDomRestyler().restyleDepedent(this, StructuralDependency);
+    // In theory BackwardsStructurualDependencies are indifferent to prepend,
+    // but it's too rare to optimize.
+    getDocument()->dynamicDomRestyler().restyleDepedent(this, BackwardsStructuralDependency);
+}
+
+void ElementImpl::backwardsStructureChanged()
+{
+    NodeBaseImpl::backwardsStructureChanged();
+
+    if (!getDocument()->renderer())
+        return; // the document is about to be destroyed
+
+    // Most selectors are not affected by append. Fire the few that are.
+    getDocument()->dynamicDomRestyler().restyleDepedent(this, BackwardsStructuralDependency);
+}
+
+void ElementImpl::attributeChanged(NodeImpl::Id id)
+{
+    if (!getDocument()->renderer())
+        return; // the document is about to be destroyed
+
+#if 0 // one-one dependencies for attributes disabled
+    getDocument()->dynamicDomRestyler().restyleDepedent(this, AttributeDependency);
+#endif
+    if (getDocument()->dynamicDomRestyler().checkDependency(id, PersonalDependency))
+        setChanged(true);
+    if (getDocument()->dynamicDomRestyler().checkDependency(id, AncestorDependency))
+        setChangedAscendentAttribute(true);
+    if (getDocument()->dynamicDomRestyler().checkDependency(id, PredecessorDependency) && parent())
+        // Any element that dependt on a predecessors attribute, also depend structurally on parent
+        parent()->structureChanged();
+}
+
 void ElementImpl::recalcStyle( StyleChange change )
 {
     // ### should go away and be done in renderobject
@@ -506,37 +617,33 @@ void ElementImpl::recalcStyle( StyleChange change )
     qDebug("recalcStyle(%d: %s, changed: %d)[%p: %s]", change, debug, changed(), this, tagName().string().latin1());
 #endif
     if ( hasParentRenderer && (change >= Inherit || changed()) ) {
-        EDisplay oldDisplay = _style ? _style->display() : NONE;
-        EPosition oldPosition = _style ? _style->position() : STATIC;
-
         RenderStyle *newStyle = getDocument()->styleSelector()->styleForElement(this);
         newStyle->ref();
         StyleChange ch = diff( _style, newStyle );
-        if ( ch != NoChange ) {
-            if (oldDisplay != newStyle->display()
-                || oldPosition != newStyle->position()) {
-                if (attached()) detach();
-                // ### uuhm, suboptimal. style gets calculated again
-                attach();
-		// attach recalulates the style for all children. No need to do it twice.
-		setChanged( false );
-		setHasChangedChild( false );
-		newStyle->deref();
-		return;
-            }
+        if (ch == Detach) {
+            if (attached()) detach();
+            // ### Suboptimal. Style gets calculated again.
+            attach();
+            // attach recalulates the style for all children. No need to do it twice.
+            setChanged( false );
+            setHasChangedChild( false );
+            newStyle->deref();
+            return;
+        }
+        else if (ch != NoChange) {
             if( m_render && newStyle ) {
-                //qDebug("--> setting style on render element bgcolor=%s", newStyle->backgroundColor().name().latin1());
                 m_render->setStyle(newStyle);
             }
         }
         newStyle->deref();
 
-       if ( change != Force) {
-            if (getDocument()->usesDescendantRules())
-                change = Force;
-            else
-                change = ch;
-        }
+        if ( change != Force)
+            change = ch;
+    }
+    // If a changed attribute has ancestor dependencies, restyle all children
+    if (changedAscendentAttribute()) {
+        change = Force;
+        setChangedAscendentAttribute(false);
     }
 
     NodeImpl *n;
@@ -557,7 +664,7 @@ bool ElementImpl::isFocusable() const
     // Only make editable elements selectable if its parent element
     // is not editable. FIXME: this is not 100% right as non-editable elements
     // within editable elements are focusable too.
-    return contentEditable() && (!parentNode() || !parentNode()->contentEditable());
+    return contentEditable() && !(parentNode() && parentNode()->contentEditable());
 }
 
 // DOM Section 1.1.1
@@ -573,7 +680,7 @@ bool ElementImpl::childAllowed( NodeImpl *newChild )
     if (isXMLElementNode() || newChild->isXMLElementNode())
 	return true;
     else
-	return checkChild(id(), newChild->id());
+	return checkChild(id(), newChild->id(), !getDocument()->inCompatMode());
 }
 
 bool ElementImpl::childTypeAllowed( unsigned short type )
@@ -592,6 +699,12 @@ bool ElementImpl::childTypeAllowed( unsigned short type )
     }
 }
 
+void ElementImpl::scrollIntoView(bool /*alignToTop*/)
+{
+    // ###
+    kdWarning() << "non-standard scrollIntoView() not implemented" << endl;
+}
+
 void ElementImpl::createDecl( )
 {
     m_styleDecls = new CSSStyleDeclarationImpl(0);
@@ -601,7 +714,7 @@ void ElementImpl::createDecl( )
     m_styleDecls->setStrictParsing( !getDocument()->inCompatMode() );
 }
 
-void ElementImpl::dispatchAttrRemovalEvent(NodeImpl::Id /*id*/, DOMStringImpl */*value*/)
+void ElementImpl::dispatchAttrRemovalEvent(NodeImpl::Id /*id*/, DOMStringImpl * /*value*/)
 {
     // ### enable this stuff again
     if (!getDocument()->hasListenerType(DocumentImpl::DOMATTRMODIFIED_LISTENER))
@@ -611,7 +724,7 @@ void ElementImpl::dispatchAttrRemovalEvent(NodeImpl::Id /*id*/, DOMStringImpl */
     //attr->value(), getDocument()->attrName(attr->id()),MutationEvent::REMOVAL),exceptioncode);
 }
 
-void ElementImpl::dispatchAttrAdditionEvent(NodeImpl::Id /*id*/, DOMStringImpl */*value*/)
+void ElementImpl::dispatchAttrAdditionEvent(NodeImpl::Id /*id*/, DOMStringImpl * /*value*/)
 {
     // ### enable this stuff again
     if (!getDocument()->hasListenerType(DocumentImpl::DOMATTRMODIFIED_LISTENER))
@@ -621,7 +734,51 @@ void ElementImpl::dispatchAttrAdditionEvent(NodeImpl::Id /*id*/, DOMStringImpl *
    //attr->value(),getDocument()->attrName(attr->id()),MutationEvent::ADDITION),exceptioncode);
 }
 
-DOMString ElementImpl::openTagStartToString() const
+void ElementImpl::updateId(DOMStringImpl* oldId, DOMStringImpl* newId)
+{
+    if (!inDocument())
+        return;
+
+    if (oldId && oldId->l)
+        removeId(DOMString(oldId).string());
+
+    if (newId && newId->l)
+        addId(DOMString(newId).string());
+}
+
+void ElementImpl::removeId(const QString& id)
+{
+  getDocument()->getElementByIdCache().remove(id, this);
+}
+
+void ElementImpl::addId(const QString& id)
+{
+  getDocument()->getElementByIdCache().add(id, this);
+}
+
+void ElementImpl::insertedIntoDocument()
+{
+    // need to do superclass processing first so inDocument() is true
+    // by the time we reach updateId
+    NodeBaseImpl::insertedIntoDocument();
+
+    if (hasID()) {
+        DOMString id = getAttribute(ATTR_ID);
+        updateId(0, id.implementation());
+    }
+}
+
+void ElementImpl::removedFromDocument()
+{
+    if (hasID()) {
+        DOMString id = getAttribute(ATTR_ID);
+        updateId(id.implementation(), 0);
+    }
+
+    NodeBaseImpl::removedFromDocument();
+}
+
+DOMString ElementImpl::openTagStartToString(bool expandurls) const
 {
     DOMString result = DOMString("<") + tagName();
 
@@ -642,7 +799,21 @@ DOMString ElementImpl::openTagStartToString() const
 		if (!attribute->value().isNull()) {
 		    result += "=\"";
 		    // FIXME: substitute entities for any instances of " or '
-		    result += attribute->value();
+		    // Expand out all urls, i.e. the src and href attributes
+		    if(expandurls && ( attribute->id() == ATTR_SRC || attribute->id() == ATTR_HREF))
+			if(getDocument()) {
+                            //We need to sanitize the urls - strip out the passwords.
+			    //FIXME:   are src=  and href=  the only places that might have a password and need to be sanitized?
+                            KURL safeURL(getDocument()->completeURL(attribute->value().string()));
+                            safeURL.setPass(QString::null);
+			    result += safeURL.htmlURL();
+			}
+		        else {
+		 	    kdWarning() << "getDocument() returned false";
+			    result += attribute->value();
+			}
+		    else
+		        result += attribute->value();
 		    result += "\"";
 		}
 	    }
@@ -651,8 +822,7 @@ DOMString ElementImpl::openTagStartToString() const
 
     return result;
 }
-
-DOMString ElementImpl::toString() const
+DOMString ElementImpl::selectionToString(NodeImpl *selectionStart, NodeImpl *selectionEnd, int startOffset, int endOffset, bool &found) const
 {
     DOMString result = openTagStartToString();
 
@@ -660,7 +830,10 @@ DOMString ElementImpl::toString() const
 	result += ">";
 
 	for (NodeImpl *child = firstChild(); child != NULL; child = child->nextSibling()) {
-	    result += child->toString();
+	    result += child->selectionToString(selectionStart, selectionEnd, startOffset, endOffset, found); // this might set found to true
+	    if(child == selectionEnd)
+	        found = true;
+	    if(found) break;
 	}
 
 	result += "</";
@@ -673,6 +846,31 @@ DOMString ElementImpl::toString() const
     return result;
 }
 
+DOMString ElementImpl::toString() const
+{
+    QString result = openTagStartToString().string(); //Accumulate in QString, since DOMString can't append well.
+
+    if (hasChildNodes()) {
+	result += ">";
+
+	for (NodeImpl *child = firstChild(); child != NULL; child = child->nextSibling()) {
+	    DOMString kid = child->toString();
+	    result += QConstString(kid.unicode(), kid.length()).string();
+	}
+
+	result += "</";
+	result += tagName().string();
+	result += ">";
+    } else if (result.length() == 1) {
+	// ensure we dont get results like < /> can happen when serialize document
+        result = "";
+    } else {
+	result += " />";
+    }
+
+    return result;
+}
+
 bool ElementImpl::contentEditable() const {
 #if 0
     DOM::CSSPrimitiveValueImpl *val = static_cast<DOM::CSSPrimitiveValueImpl *>
@@ -706,14 +904,14 @@ void ElementImpl::setContentEditable(bool enabled) {
 
 // -------------------------------------------------------------------------
 
-XMLElementImpl::XMLElementImpl(DocumentPtr *doc, NodeImpl::Id id)
+XMLElementImpl::XMLElementImpl(DocumentImpl *doc, NodeImpl::Id id)
     : ElementImpl(doc)
 {
     // Called from createElement(). In this case localName, prefix, and namespaceURI all need to be null.
     m_id = id;
 }
 
-XMLElementImpl::XMLElementImpl(DocumentPtr *doc, NodeImpl::Id id, DOMStringImpl *_prefix)
+XMLElementImpl::XMLElementImpl(DocumentImpl *doc, NodeImpl::Id id, DOMStringImpl *_prefix)
     : ElementImpl(doc)
 {
     // Called from createElementNS()
@@ -780,7 +978,7 @@ NodeImpl *NamedAttrMapImpl::getNamedItem ( NodeImpl::Id id, bool nsAware, DOMStr
     for (unsigned long i = 0; i < m_attrCount; i++) {
 	if ((m_attrs[i].id() & mask) == id) {
             // if we are called with a qualified name, filter out NS-aware elements with non-matching name.
-            if (qName && (m_attrs[i].id() & NodeImpl_IdNSMask) &&
+            if (qName && (namespacePart(m_attrs[i].id()) != defaultNamespace) &&
                 strcasecmp(m_attrs[i].name(), DOMString(qName)))
                 continue;
 	    return m_attrs[i].createAttr(m_element,m_element->docPtr());
@@ -808,16 +1006,19 @@ Node NamedAttrMapImpl::removeNamedItem ( NodeImpl::Id id, bool nsAware, DOMStrin
     for (unsigned long i = 0; i < m_attrCount; i++) {
 	if ((m_attrs[i].id() & mask) == id) {
             // if we are called with a qualified name, filter out NS-aware elements with non-matching name.
-            if (qName && (m_attrs[i].id() & NodeImpl_IdNSMask) &&
+            if (qName && (namespacePart(m_attrs[i].id()) != defaultNamespace) &&
                 strcasecmp(m_attrs[i].name(), DOMString(qName)))
                 continue;
 	    id = m_attrs[i].id();
+	    if (id == ATTR_ID)
+	       m_element->updateId(m_attrs[i].val(), 0);
 	    Node removed(m_attrs[i].createAttr(m_element,m_element->docPtr()));
 	    m_attrs[i].free();
 	    memmove(m_attrs+i,m_attrs+i+1,(m_attrCount-i-1)*sizeof(AttributeImpl));
 	    m_attrCount--;
 	    m_attrs = (AttributeImpl*)realloc(m_attrs,m_attrCount*sizeof(AttributeImpl));
 	    m_element->parseAttribute(id,0);
+	    m_element->attributeChanged(id);
 	    return removed;
 	}
     }
@@ -872,10 +1073,13 @@ Node NamedAttrMapImpl::setNamedItem ( NodeImpl* arg, bool nsAware, DOMStringImpl
     for (unsigned long i = 0; i < m_attrCount; i++) {
 	if ((m_attrs[i].id() & mask) == id) {
             // if we are called with a qualified name, filter out NS-aware elements with non-matching name.
-            if (qName && (m_attrs[i].id() & NodeImpl_IdNSMask) &&
+            if (qName && (namespacePart(m_attrs[i].id()) != defaultNamespace) &&
                 strcasecmp(m_attrs[i].name(), DOMString(qName)))
                 continue;
 	    // Attribute exists; replace it
+	    if (id == ATTR_ID)
+	       m_element->updateId(m_attrs[i].val(), attr->val());
+
 	    Node replaced = m_attrs[i].createAttr(m_element,m_element->docPtr());
 	    m_attrs[i].free();
 	    m_attrs[i].m_attrId = 0; /* "has implementation" flag */
@@ -883,6 +1087,7 @@ Node NamedAttrMapImpl::setNamedItem ( NodeImpl* arg, bool nsAware, DOMStringImpl
 	    m_attrs[i].m_data.attr->ref();
 	    attr->setElement(m_element);
 	    m_element->parseAttribute(&m_attrs[i]);
+	    m_element->attributeChanged(m_attrs[i].id());
 	    // ### dispatch mutation events
 	    return replaced;
 	}
@@ -895,7 +1100,10 @@ Node NamedAttrMapImpl::setNamedItem ( NodeImpl* arg, bool nsAware, DOMStringImpl
     m_attrs[m_attrCount-1].m_data.attr = attr;
     m_attrs[m_attrCount-1].m_data.attr->ref();
     attr->setElement(m_element);
+    if (id == ATTR_ID)
+        m_element->updateId(0, attr->val());
     m_element->parseAttribute(&m_attrs[m_attrCount-1]);
+    m_element->attributeChanged(m_attrs[m_attrCount-1].id());
     // ### dispatch mutation events
 
     return 0;
@@ -939,8 +1147,8 @@ DOMStringImpl *NamedAttrMapImpl::getValue(NodeImpl::Id id, bool nsAware, DOMStri
     for (unsigned long i = 0; i < m_attrCount; i++)
         if ((m_attrs[i].id() & mask) == id) {
             // if we are called with a qualified name, filter out NS-aware elements with non-matching name.
-            if (qName && (m_attrs[i].id() & NodeImpl_IdNSMask) &&
-                strcasecmp(m_attrs[i].name(), DOMString(qName)))
+            if (qName && (namespacePart(m_attrs[i].id()) != defaultNamespace) &&
+                strcasecmp(m_attrs[i].name(), qName))
                 continue;
             return m_attrs[i].val();
         }
@@ -966,7 +1174,7 @@ void NamedAttrMapImpl::setValue(NodeImpl::Id id, DOMStringImpl *value, DOMString
     for (unsigned long i = 0; i < m_attrCount; i++) {
 	if ((m_attrs[i].id() & mask) == mid) {
             // if we are called with a qualified name, filter out NS-aware elements with non-matching name.
-            if (qName && (m_attrs[i].id() & NodeImpl_IdNSMask) &&
+            if (qName && (namespacePart(m_attrs[i].id()) != defaultNamespace) &&
                 strcasecmp(m_attrs[i].name(), DOMString(qName)))
                 continue;
 	    if (prefix)
@@ -999,8 +1207,12 @@ void NamedAttrMapImpl::setValue(NodeImpl::Id id, DOMStringImpl *value, DOMString
         m_attrs[m_attrCount-1].m_data.attr->setHTMLCompat( !hasNS &&
                                                  m_element->getDocument()->htmlMode() != DocumentImpl::XHtml );
     }
-    if (m_element)
+    if (m_element) {
+	if (id == ATTR_ID)
+	    m_element->updateId(0, value);
 	m_element->parseAttribute(&m_attrs[m_attrCount-1]);
+	m_element->attributeChanged(m_attrs[m_attrCount-1].id());
+    }
     // ### dispatch mutation events
 }
 
@@ -1009,12 +1221,15 @@ Attr NamedAttrMapImpl::removeAttr(AttrImpl *attr)
     for (unsigned long i = 0; i < m_attrCount; i++) {
 	if (m_attrs[i].attr() == attr) {
 	    NodeImpl::Id id = m_attrs[i].id();
+	    if (id == ATTR_ID)
+	        m_element->updateId(attr->val(), 0);
 	    Node removed(m_attrs[i].createAttr(m_element,m_element->docPtr()));
 	    m_attrs[i].free();
 	    memmove(m_attrs+i,m_attrs+i+1,(m_attrCount-i-1)*sizeof(AttributeImpl));
 	    m_attrCount--;
 	    m_attrs = (AttributeImpl*)realloc(m_attrs,m_attrCount*sizeof(AttributeImpl));
 	    m_element->parseAttribute(id,0);
+	    m_element->attributeChanged(id);
 	    // ### dispatch mutation events
 	    return removed;
 	}
@@ -1037,8 +1252,11 @@ void NamedAttrMapImpl::copyAttributes(NamedAttrMapImpl *other)
 {
     assert(m_element);
     unsigned long i;
-    for (i = 0; i < m_attrCount; i++)
+    for (i = 0; i < m_attrCount; i++) {
+        if (m_attrs[i].id() == ATTR_ID)
+            m_element->updateId(m_attrs[i].val(), 0);
 	m_attrs[i].free();
+    }
     m_attrCount = other->m_attrCount;
     m_attrs = (AttributeImpl*)realloc(m_attrs,m_attrCount*sizeof(AttributeImpl));
     for (i = 0; i < m_attrCount; i++) {
@@ -1052,7 +1270,10 @@ void NamedAttrMapImpl::copyAttributes(NamedAttrMapImpl *other)
 	    m_attrs[i].m_data.attr->ref();
 	    m_attrs[i].m_data.attr->setElement(m_element);
 	}
+	if (m_attrs[i].id() == ATTR_ID)
+	   m_element->updateId(0, m_attrs[i].val());
 	m_element->parseAttribute(&m_attrs[i]);
+	m_element->attributeChanged(m_attrs[i].id());
     }
 }
 
