@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 
     Recommended reading explaining FTP details and quirks:
       http://cr.yp.to/ftp.html  (by D.J. Bernstein)
@@ -58,6 +58,7 @@
 #include <kmimemagic.h>
 #include <kmimetype.h>
 #include <ksockaddr.h>
+#include <ksocketaddress.h>
 #include <kio/ioslave_defaults.h>
 #include <kio/slaveconfig.h>
 #include <kremoteencoding.h>
@@ -134,7 +135,7 @@ KIO::filesize_t Ftp::UnknownSize = (KIO::filesize_t)-1;
 
 using namespace KIO;
 
-extern "C" { int kdemain(int argc, char **argv); }
+extern "C" { KDE_EXPORT int kdemain(int argc, char **argv); }
 
 int kdemain( int argc, char **argv )
 {
@@ -612,7 +613,7 @@ bool Ftp::ftpLogin()
       }
     }
 
-    tempbuf = "user ";
+    tempbuf = "USER ";
     tempbuf += user.latin1();
     if ( m_bUseProxy )
     {
@@ -662,7 +663,7 @@ bool Ftp::ftpLogin()
 
   // Okay, we're logged in. If this is IIS 4, switch dir listing style to Unix:
   // Thanks to jk@soegaard.net (Jens Kristian Sgaard) for this hint
-  if( ftpSendCmd("syst") && (m_iRespType == 2) )
+  if( ftpSendCmd("SYST") && (m_iRespType == 2) )
   {
     if( !strncmp( ftpResponse(0), "215 Windows_NT", 14 ) ) // should do for any version
     {
@@ -677,14 +678,14 @@ bool Ftp::ftpLogin()
     }
   }
   else
-    kdWarning(7102) << "syst failed" << endl;
+    kdWarning(7102) << "SYST failed" << endl;
 
   if ( config()->readBoolEntry ("EnableAutoLoginMacro") )
     ftpAutoLoginMacro ();
 
   // Get the current working directory
   kdDebug(7102) << "Searching for pwd" << endl;
-  if( !ftpSendCmd("pwd") || (m_iRespType != 2) )
+  if( !ftpSendCmd("PWD") || (m_iRespType != 2) )
   {
     kdDebug(7102) << "Couldn't issue pwd command" << endl;
     error( ERR_COULD_NOT_LOGIN, i18n("Could not login to %1.").arg(m_host) ); // or anything better ?
@@ -708,15 +709,14 @@ void Ftp::ftpAutoLoginMacro ()
 {
   QString macro = metaData( "autoLoginMacro" );
 
-  if ( !macro.isEmpty() )
-  {
+  if ( macro.isEmpty() )
+    return;
+
     QStringList list = QStringList::split('\n', macro);
 
-    if ( !list.isEmpty() )
-    {
       for(QStringList::Iterator it = list.begin() ; it != list.end() ; ++it )
       {
-        if ( (*it).find("init") == 0 )
+    if ( (*it).startsWith("init") )
         {
           list = QStringList::split( '\\', macro);
           it = list.begin();
@@ -733,8 +733,6 @@ void Ftp::ftpAutoLoginMacro ()
           break;
         }
       }
-    }
-  }
 }
 
 
@@ -751,6 +749,14 @@ bool Ftp::ftpSendCmd( const QCString& cmd, int maxretries )
 {
   assert(m_control != NULL);    // must have control connection socket
 
+  if ( cmd.find( '\r' ) != -1 || cmd.find( '\n' ) != -1)
+  {
+    kdWarning(7102) << "Invalid command received (contains CR or LF):"
+                    << cmd.data() << endl;
+    error( ERR_UNSUPPORTED_ACTION, m_host );
+    return false;
+  }
+
   // Don't print out the password...
   bool isPassCmd = (cmd.left(4).lower() == "pass");
   if ( !isPassCmd )
@@ -830,7 +836,6 @@ bool Ftp::ftpSendCmd( const QCString& cmd, int maxretries )
   return true;
 }
 
-
 /*
  * ftpOpenPASVDataConnection - set up data connection, using PASV mode
  *
@@ -848,6 +853,8 @@ int Ftp::ftpOpenPASVDataConnection()
   if (sa != NULL && sa->family() != PF_INET)
     return ERR_INTERNAL;       // no PASV for non-PF_INET connections
 
+  const KInetSocketAddress *sin = static_cast<const KInetSocketAddress*>(sa);
+
   if (m_extControl & pasvUnknown)
     return ERR_INTERNAL;       // already tried and got "unknown command"
 
@@ -881,13 +888,17 @@ int Ftp::ftpOpenPASVDataConnection()
   }
 
   // Make hostname and port number ...
-  QString host;
-  host.sprintf("%d.%d.%d.%d", i[0], i[1], i[2], i[3]);
   int port = i[4] << 8 | i[5];
 
+  // we ignore the host part on purpose for two reasons
+  // a) it might be wrong anyway
+  // b) it would make us being suceptible to a port scanning attack
+
   // now connect the data socket ...
   m_data = new FtpSocket("PASV");
-  m_data->setAddress(host, port);
+  m_data->setAddress(sin->nodeName(), port);
+
+  kdDebug(7102) << "Connecting to " << sin->nodeName() << " on port " << port << endl;
   return m_data->connectSocket(connectTimeout(), false);
 }
 
@@ -995,40 +1006,48 @@ int Ftp::ftpOpenDataConnection()
   assert( m_bLoggedOn );
   ftpCloseDataConnection();
 
-  int  iErrCode = 0;      // ERR_INTERNAL : try another mode
+  int  iErrCode = 0;
+  int  iErrCodePASV = 0;  // Remember error code from PASV
 
   // First try passive (EPSV & PASV) modes
   if( !config()->readBoolEntry("DisablePassiveMode", false) )
   {
     iErrCode = ftpOpenPASVDataConnection();
-    if(iErrCode != ERR_INTERNAL)
-      return iErrCode;
+    if(iErrCode == 0)
+      return 0; // success
+    iErrCodePASV = iErrCode;
     ftpCloseDataConnection();
 
     if( !config()->readBoolEntry("DisableEPSV", false) )
     {
       iErrCode = ftpOpenEPSVDataConnection();
-      if(iErrCode != ERR_INTERNAL)
-        return iErrCode;
+      if(iErrCode == 0)
+        return 0; // success
       ftpCloseDataConnection();
     }
 
     // if we sent EPSV ALL already and it was accepted, then we can't
     // use active connections any more
     if (m_extControl & epsvAllSent)
-      return ERR_INTERNAL;
+      return iErrCodePASV ? iErrCodePASV : iErrCode;
   }
 
-  iErrCode = ftpOpenEPRTDataConnection();
-  if(iErrCode != ERR_INTERNAL)
-    return iErrCode;
-  ftpCloseDataConnection();
+  if( !config()->readBoolEntry("DisableEPRT", false) )
+  {
+    iErrCode = ftpOpenEPRTDataConnection();
+    if(iErrCode == 0)
+      return 0; // success
+    ftpCloseDataConnection();
+  }
 
   // fall back to port mode
   iErrCode = ftpOpenPortDataConnection();
-  if(iErrCode != 0)
-    ftpCloseDataConnection();
-  return iErrCode;
+  if(iErrCode == 0)
+    return 0; // success
+
+  ftpCloseDataConnection();
+  // prefer to return the error code from PASV if any, since that's what should have worked in the first place
+  return iErrCodePASV ? iErrCodePASV : iErrCode;
 }
 
 /*
@@ -1051,7 +1070,7 @@ int Ftp::ftpOpenPortDataConnection()
 
   // yes, we are sure this is a KInetSocketAddress
   const KInetSocketAddress* pAddr = static_cast<const KInetSocketAddress*>(m_control->localAddress());
-  m_data->setAddress(pAddr->nodeName(), pAddr->serviceName());
+  m_data->setAddress(pAddr->nodeName(), "0");
   m_data->setAddressReusable(true);
 
   if(m_data->listen(1) < 0)
@@ -1184,7 +1203,7 @@ bool Ftp::ftpCloseCommand()
   kdDebug(7102) << "ftpCloseCommand: reading command result" << endl;
   m_bBusy = false;
 
-  if(ftpResponse(-1) <= 0 || (m_iRespType != 2) )
+  if(!ftpResponse(-1) || (m_iRespType != 2) )
   {
     kdDebug(7102) << "ftpCloseCommand: no transfer complete message" << endl;
     return false;
@@ -1740,6 +1759,11 @@ bool Ftp::ftpReadDir(FtpEntry& de)
     const char* buffer = m_data->textLine();
     kdDebug(7102) << "dir > " << buffer << endl;
 
+    //Normally the listing looks like
+    // -rw-r--r--   1 dfaure   dfaure        102 Nov  9 12:30 log
+    // but on Netware servers like ftp://ci-1.ci.pwr.wroc.pl/ it looks like (#76442)
+    // d [RWCEAFMS] Admin                     512 Oct 13  2004 PSI
+
     // we should always get the following 5 fields ...
     const char *p_access, *p_junk, *p_owner, *p_group, *p_size;
     if( (p_access = strtok((char*)buffer," ")) == 0) continue;
@@ -1748,180 +1772,183 @@ bool Ftp::ftpReadDir(FtpEntry& de)
     if( (p_group = strtok(NULL," ")) == 0) continue;
     if( (p_size  = strtok(NULL," ")) == 0) continue;
 
-    // try to parse it as a directory entry ...
+    //kdDebug(7102) << "p_access=" << p_access << " p_junk=" << p_junk << " p_owner=" << p_owner << " p_group=" << p_group << " p_size=" << p_size << endl;
+
+    de.access = 0;
+    if ( strlen( p_access ) == 1 && p_junk[0] == '[' ) { // Netware
+      de.access = S_IRWXU | S_IRWXG | S_IRWXO; // unknown -> give all permissions
+    }
+
     const char *p_date_1, *p_date_2, *p_date_3, *p_name;
 
-            // A special hack for "/dev". A listing may look like this:
-            // crw-rw-rw-   1 root     root       1,   5 Jun 29  1997 zero
-            // So we just ignore the number in front of the ",". Ok, its a hack :-)
-            if ( strchr( p_size, ',' ) != 0L )
-            {
-              //kdDebug(7102) << "Size contains a ',' -> reading size again (/dev hack)" << endl;
-              if ((p_size = strtok(NULL," ")) == 0)
-                continue;
-            }
+    // A special hack for "/dev". A listing may look like this:
+    // crw-rw-rw-   1 root     root       1,   5 Jun 29  1997 zero
+    // So we just ignore the number in front of the ",". Ok, its a hack :-)
+    if ( strchr( p_size, ',' ) != 0L )
+    {
+      //kdDebug(7102) << "Size contains a ',' -> reading size again (/dev hack)" << endl;
+      if ((p_size = strtok(NULL," ")) == 0)
+        continue;
+    }
 
-            // Check whether the size we just read was really the size
-            // or a month (this happens when the server lists no group)
-            // Test on sunsite.uio.no, for instance
-            if ( !isdigit( *p_size ) )
-            {
-              p_date_1 = p_size;
-              p_size = p_group;
-              p_group = 0;
-              //kdDebug(7102) << "Size didn't have a digit -> size=" << p_size << " date_1=" << p_date_1 << endl;
-            }
-            else
-            {
-              p_date_1 = strtok(NULL," ");
-              //kdDebug(7102) << "Size has a digit -> ok. p_date_1=" << p_date_1 << endl;
-            }
+    // Check whether the size we just read was really the size
+    // or a month (this happens when the server lists no group)
+    // Used to be the case on sunsite.uio.no, but not anymore
+    // This is needed for the Netware case, too.
+    if ( !isdigit( *p_size ) )
+    {
+      p_date_1 = p_size;
+      p_size = p_group;
+      p_group = 0;
+      //kdDebug(7102) << "Size didn't have a digit -> size=" << p_size << " date_1=" << p_date_1 << endl;
+    }
+    else
+    {
+      p_date_1 = strtok(NULL," ");
+      //kdDebug(7102) << "Size has a digit -> ok. p_date_1=" << p_date_1 << endl;
+    }
 
-            if ( p_date_1 != 0 )
-              if ((p_date_2 = strtok(NULL," ")) != 0)
-                if ((p_date_3 = strtok(NULL," ")) != 0)
-                  if ((p_name = strtok(NULL,"\r\n")) != 0)
-                  {
+    if ( p_date_1 != 0 &&
+         (p_date_2 = strtok(NULL," ")) != 0 &&
+         (p_date_3 = strtok(NULL," ")) != 0 &&
+         (p_name = strtok(NULL,"\r\n")) != 0 )
+    {
+      {
+        QCString tmp( p_name );
+        if ( p_access[0] == 'l' )
+        {
+          int i = tmp.findRev( " -> " );
+          if ( i != -1 ) {
+            de.link = remoteEncoding()->decode(p_name + i + 4);
+            tmp.truncate( i );
+          }
+          else
+            de.link = QString::null;
+        }
+        else
+          de.link = QString::null;
 
-                    {
-                       QCString tmp( p_name );
-                       if ( p_access[0] == 'l' )
-                       {
-                         int i = tmp.findRev( " -> " );
-                         if ( i != -1 ) {
-                           de.link = remoteEncoding()->decode(p_name + i + 4);
-                           tmp.truncate( i );
-                         }
-                         else
-                           de.link = QString::null;
-                       }
-                       else
-                         de.link = QString::null;
+        if ( tmp[0] == '/' ) // listing on ftp://ftp.gnupg.org/ starts with '/'
+          tmp.remove( 0, 1 );
 
-                       if ( tmp[0] == '/' ) // listing on ftp://ftp.gnupg.org/ starts with '/'
-                         tmp.remove( 0, 1 );
+        if (tmp.find('/') != -1)
+          continue; // Don't trick us!
+        // Some sites put more than one space between the date and the name
+        // e.g. ftp://ftp.uni-marburg.de/mirror/
+        de.name     = remoteEncoding()->decode(tmp.stripWhiteSpace());
+      }
 
-                       if (tmp.find('/') != -1)
-                         continue; // Don't trick us!
-                       // Some sites put more than one space between the date and the name
-                       // e.g. ftp://ftp.uni-marburg.de/mirror/
-                       de.name     = remoteEncoding()->decode(tmp.stripWhiteSpace());
-                    }
+      de.type = S_IFREG;
+      switch ( p_access[0] ) {
+      case 'd':
+        de.type = S_IFDIR;
+        break;
+      case 's':
+        de.type = S_IFSOCK;
+        break;
+      case 'b':
+        de.type = S_IFBLK;
+        break;
+      case 'c':
+        de.type = S_IFCHR;
+        break;
+      case 'l':
+        de.type = S_IFREG;
+        // we don't set S_IFLNK here.  de.link says it.
+        break;
+      default:
+        break;
+      }
 
-                    de.access = 0;
-                    de.type = S_IFREG;
-                    switch ( p_access[0] ) {
-                        case 'd':
-                            de.type = S_IFDIR;
-                            break;
-                        case 's':
-                            de.type = S_IFSOCK;
-                            break;
-                        case 'b':
-                            de.type = S_IFBLK;
-                            break;
-                        case 'c':
-                            de.type = S_IFCHR;
-                            break;
-                        case 'l':
-                            de.type = S_IFREG;
-                            // we don't set S_IFLNK here.  de.link says it.
-                            break;
-                        default:
-                            break;
-                    }
+      if ( p_access[1] == 'r' )
+        de.access |= S_IRUSR;
+      if ( p_access[2] == 'w' )
+        de.access |= S_IWUSR;
+      if ( p_access[3] == 'x' || p_access[3] == 's' )
+        de.access |= S_IXUSR;
+      if ( p_access[4] == 'r' )
+        de.access |= S_IRGRP;
+      if ( p_access[5] == 'w' )
+        de.access |= S_IWGRP;
+      if ( p_access[6] == 'x' || p_access[6] == 's' )
+        de.access |= S_IXGRP;
+      if ( p_access[7] == 'r' )
+        de.access |= S_IROTH;
+      if ( p_access[8] == 'w' )
+        de.access |= S_IWOTH;
+      if ( p_access[9] == 'x' || p_access[9] == 't' )
+        de.access |= S_IXOTH;
+      if ( p_access[3] == 's' || p_access[3] == 'S' )
+        de.access |= S_ISUID;
+      if ( p_access[6] == 's' || p_access[6] == 'S' )
+        de.access |= S_ISGID;
+      if ( p_access[9] == 't' || p_access[9] == 'T' )
+        de.access |= S_ISVTX;
 
-                    if ( p_access[1] == 'r' )
-                      de.access |= S_IRUSR;
-                    if ( p_access[2] == 'w' )
-                      de.access |= S_IWUSR;
-                    if ( p_access[3] == 'x' || p_access[3] == 's' )
-                      de.access |= S_IXUSR;
-                    if ( p_access[4] == 'r' )
-                      de.access |= S_IRGRP;
-                    if ( p_access[5] == 'w' )
-                      de.access |= S_IWGRP;
-                    if ( p_access[6] == 'x' || p_access[6] == 's' )
-                      de.access |= S_IXGRP;
-                    if ( p_access[7] == 'r' )
-                      de.access |= S_IROTH;
-                    if ( p_access[8] == 'w' )
-                      de.access |= S_IWOTH;
-                    if ( p_access[9] == 'x' || p_access[9] == 't' )
-                      de.access |= S_IXOTH;
-                    if ( p_access[3] == 's' || p_access[3] == 'S' )
-                      de.access |= S_ISUID;
-                    if ( p_access[6] == 's' || p_access[6] == 'S' )
-                      de.access |= S_ISGID;
-                    if ( p_access[9] == 't' || p_access[9] == 'T' )
-                      de.access |= S_ISVTX;
+      de.owner    = remoteEncoding()->decode(p_owner);
+      de.group    = remoteEncoding()->decode(p_group);
+      de.size     = charToLongLong(p_size);
 
-                    // maybe fromLocal8Bit would be better in some cases,
-                    // but what proves that the ftp server is in the same encoding
-                    // than the user ??
-                    de.owner    = remoteEncoding()->decode(p_owner);
-                    de.group    = remoteEncoding()->decode(p_group);
-                    de.size     = charToLongLong(p_size);
+      // Parsing the date is somewhat tricky
+      // Examples : "Oct  6 22:49", "May 13  1999"
 
-                    // Parsing the date is somewhat tricky
-                    // Examples : "Oct  6 22:49", "May 13  1999"
+      // First get current time - we need the current month and year
+      time_t currentTime = time( 0L );
+      struct tm * tmptr = gmtime( &currentTime );
+      int currentMonth = tmptr->tm_mon;
+      //kdDebug(7102) << "Current time :" << asctime( tmptr ) << endl;
+      // Reset time fields
+      tmptr->tm_isdst = -1; // We do not know anything about day saving time (of any random day of the year)
+      tmptr->tm_sec = 0;
+      tmptr->tm_min = 0;
+      tmptr->tm_hour = 0;
+      // Get day number (always second field)
+      tmptr->tm_mday = atoi( p_date_2 );
+      // Get month from first field
+      // NOTE : no, we don't want to use KLocale here
+      // It seems all FTP servers use the English way
+      //kdDebug(7102) << "Looking for month " << p_date_1 << endl;
+      static const char * s_months[12] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
+                                           "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
+      for ( int c = 0 ; c < 12 ; c ++ )
+        if ( !strcmp( p_date_1, s_months[c]) )
+        {
+          //kdDebug(7102) << "Found month " << c << " for " << p_date_1 << endl;
+          tmptr->tm_mon = c;
+          break;
+        }
 
-                    // First get current time - we need the current month and year
-                    time_t currentTime = time( 0L );
-                    struct tm * tmptr = gmtime( &currentTime );
-                    int currentMonth = tmptr->tm_mon;
-                    //kdDebug(7102) << "Current time :" << asctime( tmptr ) << endl;
-                    // Reset time fields
-                    tmptr->tm_sec = 0;
-                    tmptr->tm_min = 0;
-                    tmptr->tm_hour = 0;
-                    // Get day number (always second field)
-                    tmptr->tm_mday = atoi( p_date_2 );
-                    // Get month from first field
-                    // NOTE : no, we don't want to use KLocale here
-                    // It seems all FTP servers use the English way
-                    //kdDebug(7102) << "Looking for month " << p_date_1 << endl;
-                    static const char * s_months[12] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
-                                                       "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
-                    for ( int c = 0 ; c < 12 ; c ++ )
-                      if ( !strcmp( p_date_1, s_months[c]) )
-                      {
-                        //kdDebug(7102) << "Found month " << c << " for " << p_date_1 << endl;
-                        tmptr->tm_mon = c;
-                        break;
-                      }
+      // Parse third field
+      if ( strlen( p_date_3 ) == 4 ) // 4 digits, looks like a year
+        tmptr->tm_year = atoi( p_date_3 ) - 1900;
+      else
+      {
+        // otherwise, the year is implicit
+        // according to man ls, this happens when it is between than 6 months
+        // old and 1 hour in the future.
+        // So the year is : current year if tm_mon <= currentMonth+1
+        // otherwise current year minus one
+        // (The +1 is a security for the "+1 hour" at the end of the month issue)
+        if ( tmptr->tm_mon > currentMonth + 1 )
+          tmptr->tm_year--;
 
-                    // Parse third field
-                    if ( strlen( p_date_3 ) == 4 ) // 4 digits, looks like a year
-                      tmptr->tm_year = atoi( p_date_3 ) - 1900;
-                    else
-                    {
-                      // otherwise, the year is implicit
-                      // according to man ls, this happens when it is between than 6 months
-                      // old and 1 hour in the future.
-                      // So the year is : current year if tm_mon <= currentMonth+1
-                      // otherwise current year minus one
-                      // (The +1 is a security for the "+1 hour" at the end of the month issue)
-                      if ( tmptr->tm_mon > currentMonth + 1 )
-                        tmptr->tm_year--;
+        // and p_date_3 contains probably a time
+        char * semicolon;
+        if ( ( semicolon = (char*)strchr( p_date_3, ':' ) ) )
+        {
+          *semicolon = '\0';
+          tmptr->tm_min = atoi( semicolon + 1 );
+          tmptr->tm_hour = atoi( p_date_3 );
+        }
+        else
+          kdWarning(7102) << "Can't parse third field " << p_date_3 << endl;
+      }
 
-                      // and p_date_3 contains probably a time
-                      char * semicolon;
-                      if ( ( semicolon = (char*)strchr( p_date_3, ':' ) ) )
-                      {
-                        *semicolon = '\0';
-                        tmptr->tm_min = atoi( semicolon + 1 );
-                        tmptr->tm_hour = atoi( p_date_3 );
-                      }
-                      else
-                        kdWarning(7102) << "Can't parse third field " << p_date_3 << endl;
-                    }
-
-                    //kdDebug(7102) << asctime( tmptr ) << endl;
-                    de.date = mktime( tmptr );
-                    return true;
-                  }
-  }
+      //kdDebug(7102) << asctime( tmptr ) << endl;
+      de.date = mktime( tmptr );
+      return true;
+    }
+  } // line invalid, loop to get another line
   return false;
 }
 
@@ -1977,6 +2004,7 @@ Ftp::StatusCode Ftp::ftpGet(int& iError, int iCopyFile, const KURL& url, KIO::fi
   {
     const char* psz = strrchr( ftpResponse(4), '(' );
     if(psz) m_size = charToLongLong(psz+1);
+    if (!m_size) m_size = UnknownSize;
   }
 
   KIO::filesize_t bytesLeft = 0;
@@ -2032,11 +2060,24 @@ Ftp::StatusCode Ftp::ftpGet(int& iError, int iCopyFile, const KURL& url, KIO::fi
     if(!mimetypeEmitted)
     {
       mimetypeEmitted = true;
-      array.setRawData(buffer, n);
-      KMimeMagicResult * result = KMimeMagic::self()->findBufferFileType(array, url.fileName());
-      array.resetRawData(buffer, n);
-      kdDebug(7102) << "ftpGet: Emitting mimetype " << result->mimeType() << endl;
-      mimeType( result->mimeType() );
+
+      // We need a KMimeType::findByNameAndContent(data,filename)
+      // For now we do: find by extension, and if not found (or extension not reliable)
+      // then find by content.
+      bool accurate = false;
+      KMimeType::Ptr mime = KMimeType::findByURL( url, 0, false, true, &accurate );
+      if ( !mime || mime->name() == KMimeType::defaultMimeType()
+           || !accurate )
+      {
+        array.setRawData(buffer, n);
+        KMimeMagicResult * result = KMimeMagic::self()->findBufferFileType(array, url.fileName());
+        array.resetRawData(buffer, n);
+        if ( result->mimeType() != KMimeType::defaultMimeType() )
+          mime = KMimeType::mimeType( result->mimeType() );
+      }
+
+      kdDebug(7102) << "ftpGet: Emitting mimetype " << mime->name() << endl;
+      mimeType( mime->name() );
       if( m_size != UnknownSize )	// Emit total size AFTER mimetype
         totalSize( m_size );
     }
@@ -2350,6 +2391,7 @@ bool Ftp::ftpSize( const QString & path, char mode )
   if(!psz)
     return false;
   m_size = charToLongLong(psz);
+  if (!m_size) m_size = UnknownSize;
   return true;
 }
 
@@ -2432,8 +2474,10 @@ void Ftp::copy( const KURL &src, const KURL &dest, int permissions, bool overwri
     cs = ftpCopyGet(iError, iCopyFile, sCopyFile, src, permissions, overwrite);
     if( cs == statusServerError ) sCopyFile = src.url();
   }
-  else
-    iError = ERR_UNSUPPORTED_ACTION;
+  else {
+    error( ERR_UNSUPPORTED_ACTION, QString::null );
+    return;
+  }
 
   // perform clean-ups and report error (if any)
   if(iCopyFile != -1)
@@ -2505,8 +2549,13 @@ Ftp::StatusCode Ftp::ftpCopyGet(int& iError, int& iCopyFile, const QString sCopy
   QCString sPart = QFile::encodeName(sCopyFile + ".part");
   bool bResume = false;
   bool bPartExists = (KDE_stat( sPart.data(), &buff ) != -1);
-  bool bMarkPartial = config()->readBoolEntry("MarkPartial", true);
-  if(bMarkPartial && bPartExists && buff.st_size > 0)
+  const bool bMarkPartial = config()->readBoolEntry("MarkPartial", true);
+
+  if(!bMarkPartial)
+  {
+    sPart = QFile::encodeName(sCopyFile);
+  }
+  else if(bPartExists && buff.st_size > 0)
   { // must not be a folder! please fix a similar bug in kio_file!!
     if(S_ISDIR(buff.st_mode))
     {
