@@ -23,7 +23,7 @@
 //----------------------------------------------------------------------------
 //
 // KDE File Manager -- HTTP Cookies
-// $Id: kcookiejar.cpp 452328 2005-08-23 01:48:51Z mueller $
+// $Id: kcookiejar.cpp 709516 2007-09-07 17:53:18Z adawit $
 
 //
 // The cookie protocol is a mess. RFC2109 is a joke since nobody seems to
@@ -82,6 +82,7 @@
 
 #define MAX_COOKIES_PER_HOST 25
 #define READ_BUFFER_SIZE 8192
+#define IP_ADDRESS_EXPRESSION "(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)"
 
 // Note with respect to QString::fromLatin1( )
 // Cookies are stored as 8 bit data and passed to kio_http as
@@ -339,6 +340,8 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long win
     bool secureRequest = (_url.find( L1("https://"), 0, false) == 0 ||
                           _url.find( L1("webdavs://"), 0, false) == 0);
 
+    kdDebug(7104) << "findCookies: URL= " << _url << ", secure = " << secureRequest << endl;
+
     extractDomains(fqdn, domains);
 
     KHttpCookieList allCookies;
@@ -366,19 +369,16 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long win
        if (cookieList->getAdvice() != KCookieDunno)
           advice = cookieList->getAdvice();
 
-       // Do not send cookies for this domain if policy is set to reject
-       // and we are not setup to automatically accept all cookies as
-       // session cookies...
-       if (advice == KCookieReject &&
-           !(m_ignoreCookieExpirationDate && m_autoAcceptSessionCookies))
-       {
-          if (it == domains.end())
-             break; // Finished.
-          continue;
-       }
-
        for ( cookie=cookieList->first(); cookie != 0; cookie=cookieList->next() )
        {
+          // If the we are setup to automatically accept all session cookies and to
+          // treat all cookies as session cookies or the current cookie is a session
+          // cookie, then send the cookie back regardless of either policy.
+          if (advice == KCookieReject &&
+              !(m_autoAcceptSessionCookies && 
+                (m_ignoreCookieExpirationDate || cookie->expireDate() == 0)))
+              continue;
+
           if (!cookie->match(fqdn, domains, path))
              continue;
 
@@ -403,7 +403,7 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long win
           {
              cookie->windowIds().append(windowId);
           }
-          
+
           if (it == domains.end()) // Only needed when processing pending cookies
              removeDuplicateFromList(&allCookies, cookie);
 
@@ -566,7 +566,7 @@ QString KCookieJar::stripDomain( KHttpCookiePtr cookiePtr)
     if (cookiePtr->domain().isEmpty())
        stripDomain( cookiePtr->host(), domain);
     else
-       domain = cookiePtr->domain();
+       stripDomain (cookiePtr->domain(), domain);
     return domain;
 }
 
@@ -620,16 +620,7 @@ void KCookieJar::extractDomains(const QString &_fqdn,
     // Return numeric IPv4 addresses as is...
     if ((_fqdn[0] >= '0') && (_fqdn[0] <= '9'))
     {
-       bool allNumeric = true;
-       for(int i = _fqdn.length(); i--;)
-       {
-          if (!strchr("0123456789:.", _fqdn[i].latin1()))
-          {
-             allNumeric = false;
-             break;
-          }
-       }
-       if (allNumeric)
+       if (_fqdn.find(QRegExp(IP_ADDRESS_EXPRESSION)) > -1)
        {
           _domains.append( _fqdn );
           return;
@@ -646,13 +637,13 @@ void KCookieJar::extractDomains(const QString &_fqdn,
 
        if (partList.count() == 1)
          break; // We only have a TLD left.
-       
+
        if ((partList.count() == 2) && (m_twoLevelTLD[partList[1].lower()]))
        {
           // This domain uses two-level TLDs in the form xxxx.yy
           break;
        }
-       
+
        if ((partList.count() == 2) && (partList[1].length() == 2))
        {
           // If this is a TLD, we should stop. (e.g. co.uk)
@@ -679,6 +670,37 @@ void KCookieJar::extractDomains(const QString &_fqdn,
     _domains.prepend( _fqdn );
 }
 
+
+/*
+   Changes dates in from the following format
+
+      Wed Sep 12 07:00:00 2007 GMT
+   to
+      Wed Sep 12 2007 07:00:00 GMT
+
+   to allow KRFCDate::parseDate to properly parse expiration date formats 
+   used in cookies by some servers such as amazon.com. See BR# 145244.
+*/
+static QString fixupDateTime(const QString& dt)
+{
+  const int index = dt.find(QRegExp("[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}"));
+
+  if (index > -1)
+  {
+    QStringList dateStrList = QStringList::split(' ', dt.mid(index));
+    if (dateStrList.count() > 1)
+    {
+      QString date = dateStrList[0];
+      dateStrList[0] = dateStrList[1];
+      dateStrList[1] = date;
+      date = dt;
+      return date.replace(index, date.length(), dateStrList.join(" "));
+    }
+  }
+
+  return dt;
+}
+
 //
 // This function parses cookie_headers and returns a linked list of
 // KHttpCookie objects for all cookies found in cookie_headers.
@@ -804,6 +826,11 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
             {
                 // Parse brain-dead netscape cookie-format
                 lastCookie->mExpireDate = KRFCDate::parseDate(Value);
+
+                // Workaround for servers that send the expiration date in
+                // 'Wed Sep 12 07:00:00 2007 GMT' format. See BR# 145244.
+                if (lastCookie->mExpireDate == 0)
+                  lastCookie->mExpireDate = KRFCDate::parseDate(fixupDateTime(Value));
             }
             else if (cName == "path")
             {
@@ -981,44 +1008,27 @@ void KCookieJar::addCookie(KHttpCookiePtr &cookiePtr)
 //
 KCookieAdvice KCookieJar::cookieAdvice(KHttpCookiePtr cookiePtr)
 {
-    QStringList domains;
-
     if (m_rejectCrossDomainCookies && cookiePtr->isCrossDomain())
        return KCookieReject;
 
+    QStringList domains;
+
+    extractDomains(cookiePtr->host(), domains);
+
+    // If the cookie specifies a domain, check whether it is valid. Otherwise,
+    // accept the cookie anyways but removes the domain="" value to prevent
+    // cross-site cookie injection.
+    if (!cookiePtr->domain().isEmpty())
+    {
+      if (!domains.contains(cookiePtr->domain()) && 
+          !cookiePtr->domain().endsWith("."+cookiePtr->host()))
+          cookiePtr->fixDomain(QString::null);
+    }
+
     if (m_autoAcceptSessionCookies && (cookiePtr->expireDate() == 0 ||
         m_ignoreCookieExpirationDate))
        return KCookieAccept;
 
-    extractDomains(cookiePtr->host(), domains);
-
-    // If the cookie specifies a domain, check whether it is valid and
-    // correct otherwise.
-    if (!cookiePtr->domain().isEmpty())
-    {
-       bool valid = false;
-
-       // This checks whether the cookie is valid based on
-       // what ::extractDomains returns
-       if (!valid)
-       {
-          if (domains.contains(cookiePtr->domain()))
-             valid = true;
-       }
-
-       if (!valid)
-       {
-          // Maybe it points to a sub-domain
-          if (cookiePtr->domain().endsWith("."+cookiePtr->host()))
-             valid = true;
-       }
-
-       if (!valid)
-       {
-          cookiePtr->fixDomain(QString::null);
-       }
-    }
-
     KCookieAdvice advice = KCookieDunno;
     bool isFQDN = true; // First is FQDN
     QStringList::Iterator it = domains.begin(); // Start with FQDN which first in the list.
